package org.frido.todo.bl;

import java.io.*;
import java.util.Hashtable;
import java.sql.Date;
import ojb.odmg.*;
import org.odmg.*;
import ojb.broker.SequenceManager;

public class InsertToDoItemBLCommand extends BLCommandBase {

	public InsertToDoItemBLCommand()
	{
		super();
	}

	public void executeHook() throws BLException
	{
	    // Get persistence implementation.
	    Implementation odmg = getImplementation();
	    // this will be our new object
	    ToDoItem toDoItem  = new ToDoItem();
	    toDoItem.setShortdescription((String)getAttribute("shortdescription"));
	    toDoItem.setDescription((String)getAttribute("description"));
	    toDoItem.setPriority(((Integer)getAttribute("priority")).intValue());
	    toDoItem.setStartdate((Date)getAttribute("startdate"));
	    toDoItem.setEnddate((Date)getAttribute("enddate"));    
	    
	    // now perform persistence operations
	    Transaction tx = null;
	    try
		{
		    // 1. open transaction
		    tx = odmg.newTransaction();
		    tx.begin();
		    
		    // 2. we need a unique id for the new Object.
		    toDoItem.setId(SequenceManager.getUniqueId(ToDoItem.class, "id"));
		    
		    // 3. create a unique name for the object to perfom named root lookups later;
		    String name = "toDoItem_" + toDoItem.getId();
		    
		    // 4. bind new object to this name for later reference:
		    odmg.getDatabase(null).makePersistent(toDoItem); 
		    // temporary workaround for a bug in bind
		    odmg.getDatabase(null).bind(toDoItem, name);

		    // 5. commit transaction
		    tx.commit();
		}
	    catch (ODMGException ex)
		{
		    // if something went wrong: rollback
		    tx.abort();
		    System.out.println(ex.getMessage());
		    ex.printStackTrace();
		    throw new BLException("Insert ToDoItem failed.");
		}
	    
	}
}
