package org.frido.webtodo.action;


import java.io.IOException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.text.ParseException;
import java.util.Hashtable;
import java.sql.Date;
import java.util.Locale;
import javax.servlet.RequestDispatcher;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import javax.servlet.http.HttpServletResponse;
import org.apache.struts.action.Action;
import org.apache.struts.action.ActionError;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionServlet;
import org.apache.struts.util.MessageResources;
import org.frido.webtodo.form.ToDoItemForm;
import org.frido.todo.bl.*;



public final class InsertToDoItemAction extends Action {


    // --------------------------------------------------------- Public Methods


    public ActionForward perform(ActionMapping mapping,
				 ActionForm form,
				 HttpServletRequest request,
				 HttpServletResponse response)
	throws IOException, ServletException {

	// Extract attributes we will need
	Locale locale = getLocale(request);
	MessageResources messages = getResources();

	// Validate the request parameters specified by the user
	ToDoItemForm toDoItemForm = (ToDoItemForm) form;

	ActionErrors errors = new ActionErrors();
	String shortDescription = toDoItemForm.getShortdescription();
	String description = toDoItemForm.getDescription();
	String priority = toDoItemForm.getPriority();
	String startdate = toDoItemForm.getStartdate();
	String enddate = toDoItemForm.getEnddate();
	Integer priorityAsInteger = null;
        Date   startdateAsDate = null;
	Date   enddateAsDate = null;
	DateFormat dateFormat = new SimpleDateFormat("dd.MM.yyyy");

        // Validate short description
	if ((shortDescription == null) ||
	    (shortDescription.trim().equals(""))
           )
	    errors.add(ActionErrors.GLOBAL_ERROR,
		       new ActionError("error.toDoItemForm.fillShortDescription"));

	// No validation needed for description

	// Validate priority
	try {
	    priorityAsInteger = Integer.valueOf(priority);
	    if (priorityAsInteger.intValue() <= 0)
		errors.add(ActionErrors.GLOBAL_ERROR,
			   new ActionError("error.toDoItemForm.wrongPriority"));
	}
	catch(NumberFormatException e){
	    errors.add(ActionErrors.GLOBAL_ERROR,
		       new ActionError("error.toDoItemForm.wrongPriority"));
	}

	// Validate startdate
	try {
	    startdateAsDate = new Date(dateFormat.parse(startdate).getTime());
	}
	catch(ParseException e){
	    errors.add(ActionErrors.GLOBAL_ERROR,
		       new ActionError("error.toDoItemForm.wrongStartdate"));
	}

	// Validate enddate
	try {
	    enddateAsDate = new Date(dateFormat.parse(enddate).getTime());
	}
	catch(ParseException e){
	    errors.add(ActionErrors.GLOBAL_ERROR,
		       new ActionError("error.toDoItemForm.wrongEnddate"));
	}

	// Report any errors we have discovered back to the original form
	if (!errors.empty()) {
	    saveErrors(request, errors);
	    return (new ActionForward(mapping.getInput()));
	}

	// Now perform the real business logic
	try {
	    InsertToDoItemBLCommand cmd = new InsertToDoItemBLCommand();
	    cmd.setAttribute("shortdescription",shortDescription);
	    cmd.setAttribute("description",description);	
	    cmd.setAttribute("priority",priorityAsInteger);	
	    cmd.setAttribute("startdate",startdateAsDate);	
	    cmd.setAttribute("enddate",enddateAsDate);	

	    cmd.execute();

	}
	catch(BLException e){
	   errors.add(ActionErrors.GLOBAL_ERROR,
		       new ActionError("error.insertToDoItemAction.failed"));
	   getServlet().log("error.insertToDoItemAction.failed",e);
	   return (mapping.findForward("failure")); 
	}


	// Forward control to the specified success URI
	return (mapping.findForward("success"));

    }


}
