package org.frido.webtodo.action;


import java.io.IOException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.text.ParseException;
import java.util.Hashtable;
import java.sql.Date;
import java.util.Locale;
import javax.servlet.RequestDispatcher;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import javax.servlet.http.HttpServletResponse;
import org.apache.struts.action.Action;
import org.apache.struts.action.ActionError;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionServlet;
import org.apache.struts.actions.DispatchAction;
import org.apache.struts.util.MessageResources;
import org.frido.webtodo.form.ToDoItemForm;
import org.frido.todo.bl.*;



public final class UpdateToDoItemAction extends DispatchAction {


    // --------------------------------------------------------- Public Methods


    public ActionForward update(ActionMapping mapping,
				ActionForm form,
				HttpServletRequest request,
				HttpServletResponse response)
	throws IOException, ServletException {

	// Extract attributes we will need
	Locale locale = getLocale(request);
	MessageResources messages = getResources();

	// Validate the request parameters specified by the user
	ToDoItemForm toDoItemForm = (ToDoItemForm) form;

	ActionErrors errors = new ActionErrors();
	String id = toDoItemForm.getId();
	String shortDescription = toDoItemForm.getShortdescription();
	String description = toDoItemForm.getDescription();
	String priority = toDoItemForm.getPriority();
	String startdate = toDoItemForm.getStartdate();
	String enddate = toDoItemForm.getEnddate();
	Integer idAsInteger =  null;
	Integer priorityAsInteger = null;
        Date   startdateAsDate = null;
	Date   enddateAsDate = null;
	DateFormat dateFormat = new SimpleDateFormat("dd.MM.yyyy");

        // FIXME: Same code as in InsertToDoItemAction.
	// Moe to common superclass.


	// Validate id
	try {
	    idAsInteger = Integer.valueOf(id);
	}
	catch(NumberFormatException e){
	    errors.add(ActionErrors.GLOBAL_ERROR,
		       new ActionError("error.toDoItemForm.wrongId"));
	}


        // Validate short description
	if ((shortDescription == null) ||
	    (shortDescription.trim().equals(""))
           )
	    errors.add(ActionErrors.GLOBAL_ERROR,
		       new ActionError("error.toDoItemForm.fillShortDescription"));

	// No validation needed for description

	// Validate priority
	try {
	    priorityAsInteger = Integer.valueOf(priority);
	    if (priorityAsInteger.intValue() <= 0)
		errors.add(ActionErrors.GLOBAL_ERROR,
			   new ActionError("error.toDoItemForm.wrongPriority"));
	}
	catch(NumberFormatException e){
	    errors.add(ActionErrors.GLOBAL_ERROR,
		       new ActionError("error.toDoItemForm.wrongPriority"));
	}

	// Validate startdate
	try {
	    startdateAsDate = new Date(dateFormat.parse(startdate).getTime());
	}
	catch(ParseException e){
	    errors.add(ActionErrors.GLOBAL_ERROR,
		       new ActionError("error.toDoItemForm.wrongStartdate"));
	}

	// Validate enddate
	try {
	    enddateAsDate = new Date(dateFormat.parse(enddate).getTime());
	}
	catch(ParseException e){
	    errors.add(ActionErrors.GLOBAL_ERROR,
		       new ActionError("error.toDoItemForm.wrongEnddate"));
	}

	// Report any errors we have discovered back to the original form
	if (!errors.empty()) {
	    saveErrors(request, errors);
	    return (new ActionForward(mapping.getInput()));
	}

	// Now perform the real business logic
	try {
	    UpdateToDoItemBLCommand cmd = new UpdateToDoItemBLCommand();
	    cmd.setAttribute("id",idAsInteger);
	    cmd.setAttribute("shortdescription",shortDescription);
	    cmd.setAttribute("description",description);	
	    cmd.setAttribute("priority",priorityAsInteger);	
	    cmd.setAttribute("startdate",startdateAsDate);	
	    cmd.setAttribute("enddate",enddateAsDate);	

	    cmd.execute();
	}
	catch(BLException e){
	   errors.add(ActionErrors.GLOBAL_ERROR,
		       new ActionError("error.updateToDoItemAction.failed"));
	   getServlet().log("error.updateToDoItemAction.failed",e);
	   return (mapping.findForward("failure")); 
	}


	// Forward control to the specified success URI
	return (mapping.findForward("successupdate"));

    }

    public ActionForward get(ActionMapping mapping,
				 ActionForm form,
				 HttpServletRequest request,
				 HttpServletResponse response)
	throws IOException, ServletException {

	// Extract attributes we will need
	Locale locale = getLocale(request);
	MessageResources messages = getResources();

	// Validate the request parameters specified by the user
	ToDoItemForm toDoItemForm = (ToDoItemForm) form;

	ActionErrors errors = new ActionErrors();
	String id = toDoItemForm.getId();
	Integer idAsInteger =  null;
	DateFormat dateFormat = new SimpleDateFormat("dd.MM.yyyy");

	// Validate id
	try {
	    idAsInteger = Integer.valueOf(id);
	}
	catch(NumberFormatException e){
	    errors.add(ActionErrors.GLOBAL_ERROR,
		       new ActionError("error.deleteDoItemForm.wrongId"));
	}


	// Report any errors we have discovered back to the original form
	if (!errors.empty()) {
	    saveErrors(request, errors);
	    return (new ActionForward(mapping.getInput()));
	}

	// Now perform the real business logic
	try {
	    GetToDoItemBLCommand cmd = new GetToDoItemBLCommand();
	    cmd.setAttribute("id",idAsInteger);	

	    cmd.execute();

	    // id will not be set. It should never change.
	    ToDoItem toDoItem = (ToDoItem) cmd.getAttribute("toDoItem");
	    toDoItemForm.setShortdescription(toDoItem.getShortdescription());
	    toDoItemForm.setDescription(toDoItem.getDescription());
	    toDoItemForm.setPriority(Integer.toString(toDoItem.getPriority()));
	    toDoItemForm.setStartdate(dateFormat.format(toDoItem.getStartdate()));
	    toDoItemForm.setEnddate(dateFormat.format(toDoItem.getEnddate()));

	    return (mapping.findForward("successget")); 

	}
	catch(BLException e){
	   errors.add(ActionErrors.GLOBAL_ERROR,
		       new ActionError("error.getToDoItemAction.failed"));
	   getServlet().log("error.getToDoItemAction.failed",e);
	   return (mapping.findForward("failure")); 
	}
    }
}






